@@ -28,13 +28,16 @@ __authors__ = ["J. Garriga"]
 __license__ = "MIT"
 __date__ = "15/05/2020"
 
-import os
-import glob
 import copy
-import numpy
+import glob
 import logging
-import h5py
+import os
+import threading
+from enum import IntEnum
+
 import cv2
+import h5py
+import numpy
 from sklearn import decomposition
 
 from silx.io import utils
@@ -71,6 +74,16 @@ for key, value in _METADATA_TYPES.items():
     _METADATA_TYPES_I[value] = key
 
 
+class Operation(IntEnum):
+    """
+    Flags for different operations in Dataset
+    """
+    PARTITION = 0
+    BS = 1
+    HP = 2
+    SHIFT = 3
+
+
 class Dataset():
     """Class to define a dataset from a series of data files.
 
@@ -99,6 +112,8 @@ class Dataset():
         self._frames_intensity = []
         self._dir = _dir
         self._treated = treated
+        self._lock = threading.Lock()
+        self.operations_state = numpy.zeros(len(Operation))
         if not os.path.isdir(self.dir + "/" + treated):
             os.mkdir(self.dir + "/" + treated)
 
@@ -133,6 +148,21 @@ class Dataset():
 
             self._data = Data(numpy.array(data_urls), metadata=metadata, in_memory=self._in_memory)
 
+    def stop_operation(self, operation):
+        """
+        Method used for cases where threads are created to apply functions to the dataset.
+        If method is called, the flag concerning the stop is set to 0 so that if the concerned
+        operation is running in another thread it knows to stop.
+
+        :param int operation: operation to stop
+        :type int: Union[int, `Operation`]
+        """
+        if self.operations_state[operation]:
+            self._lock.acquire()
+            self.operations_state[operation] = 0
+            self._lock.release()
+            self.get_data().stop_operation(operation)
+
     @property
     def dir(self):
         return self._dir
@@ -148,14 +178,24 @@ class Dataset():
         """
         _logger.info("Computing intensity per frame")
         io_utils.advancement_display(0, self.nframes, "Computing intensity")
-        self._frames_intensity = []
+        frames_intensity = []
+        self._lock.acquire()
+        self.operations_state[Operation.PARTITION] = 1
+        self._lock.release()
+
         for i in range(self.nframes):
-            self._frames_intensity += \
+            if not self.operations_state[Operation.PARTITION]:
+                return
+            frames_intensity += \
                 [cv2.GaussianBlur(self.get_data(i), kernel, sigma).var()]
             io_utils.advancement_display(i + 1, self.nframes, "Computing intensity")
+        self._frames_intensity = frames_intensity
+        self._lock.acquire()
+        self.operations_state[Operation.PARTITION] = 0
+        self._lock.release()
         return self._frames_intensity
 
-    def partition_by_intensity(self, num_bins=1):
+    def partition_by_intensity(self, bins=None, num_bins=1):
         """
         Function that computes the data from the set of urls.
         If the filter_data flag is activated it filters the data following the next:
@@ -168,11 +208,17 @@ class Dataset():
         :param int num_bins: Number of bins to use as threshold.
         """
         frames_intensity = self._frames_intensity if self._frames_intensity else self.compute_frames_intensity()
-        values, bins = numpy.histogram(frames_intensity, self.nframes)
+        if frames_intensity is None:
+            return
+        values, bins = numpy.histogram(frames_intensity, self.nframes if bins is None else bins)
         threshold = numpy.array(frames_intensity) >= bins[num_bins]
-
         return numpy.flatnonzero(threshold), numpy.flatnonzero(~threshold)
 
+    @property
+    def in_memory(self):
+        return self._in_memory
+
+    @in_memory.setter
     def in_memory(self, in_memory):
         """
         Removes data from memory and sets that from now on data will be read from disk.
@@ -220,6 +266,23 @@ class Dataset():
         else:
             return len(self.data.flatten())
 
+    def to_memory(self, indices):
+        """
+        Method to load only part of the data into memory.
+        Returns a new dataset with the data corresponding to given indices into memory.
+        The new indices array has to be given, if all the data has to be set into
+        memory please set `in_memory` to True instead, this way no new dataset will be
+        created.
+
+        :param array_like indices: Indices of the new dataset.
+        """
+        if not self._in_memory:
+            data = self.get_data(indices)
+            new_data = Data(data.urls, data.metadata, True)
+        else:
+            new_data = self.get_data(indices)
+        return Dataset(_dir=self._dir, data=new_data, dims=self.__dims, in_memory=True)
+
     @property
     def dims(self):
         return self.__dims
@@ -356,13 +419,16 @@ class Dataset():
             the new urls.
         :rtype: Dataset
         """
-
         bs_dir = self.dir + "/" + self.treated + "/bs"
         if not os.path.isdir(bs_dir):
             os.mkdir(bs_dir)
 
         data = self.get_data(indices)
 
+        self._lock.acquire()
+        self.operations_state[Operation.BS] = 1
+        self._lock.release()
+
         if background is None:
             bg_data = data
             if indices is None:
@@ -389,6 +455,8 @@ class Dataset():
                 else:
                     io_utils.advancement_display(0, len(bg_data), "Computing mean image")
                     for i in range(len(bg_data)):
+                        if not self.operations_state[Operation.BS]:
+                            return
                         bg = img2img_mean(bg_data[i], bg, i)
                         io_utils.advancement_display(i + 1, len(bg_data), "Computing mean image")
             elif method == Method.median:
@@ -398,21 +466,34 @@ class Dataset():
                     start = [0, 0]
                     img = data[0]
                     bg = numpy.empty(img.shape, img.dtype)
-                    chunks = int(numpy.ceil(img.shape[0] / chunk_shape[0]))
-                    io_utils.advancement_display(0, chunks, "Computing median image")
-                    for i in range(chunks):
-                        for j in range(int(numpy.ceil(img.shape[1] / chunk_shape[1]))):
+                    chunks_1 = int(numpy.ceil(img.shape[0] / chunk_shape[0]))
+                    chunks_2 = int(numpy.ceil(img.shape[1] / chunk_shape[1]))
+                    io_utils.advancement_display(0, chunks_1 * chunks_2 * len(bg_data), "Computing median image")
+                    for i in range(chunks_1):
+                        if not self.operations_state[Operation.BS]:
+                            return
+                        for j in range(chunks_2):
                             c_images = []
-                            for img in bg_data:
+                            for k, img in enumerate(bg_data):
                                 c_images += [chunk_image(img, start, chunk_shape)]
+                                io_utils.advancement_display(
+                                    i * chunks_2 * len(bg_data) + j * len(bg_data) + k + 1,
+                                    chunks_1 * chunks_2 * len(bg_data),
+                                    "Computing median image")
                             numpy.median(c_images, out=bg[start[0]: start[0] + chunk_shape[0],
                                          start[1]:start[1] + chunk_shape[1]], axis=0)
                             start[1] = chunk_shape[1] * (j + 1)
-                        io_utils.advancement_display(i + 1, chunks, "Computing median image")
                         start[0] = chunk_shape[0] * (i + 1)
                         start[1] = 0
             urls = data.apply_funcs([(background_subtraction_2D, [bg])],
-                                    save=bs_dir + "/data_bs", text="Applying background subtraction")
+                                    save=bs_dir + "/data_bs", text="Applying background subtraction",
+                                    operation=Operation.BS)
+            if urls is None:
+                return
+
+        self._lock.acquire()
+        self.operations_state[Operation.BS] = 0
+        self._lock.release()
 
         # Set urls as shape and dimension of original urls.
         if indices is not None:
@@ -453,7 +534,10 @@ class Dataset():
             urls = new_data.urls
         else:
             urls = data.apply_funcs([(hot_pixel_removal_2D, [kernel])],
-                                    save=_dir + "/data_hp", text="Applying hot pixel removal")
+                                    save=_dir + "/data_hp", text="Applying hot pixel removal",
+                                    operation=Operation.HP)
+            if urls is None:
+                return
 
         if indices is not None:
             new_urls = numpy.array(self.get_data().urls, dtype=object)
@@ -494,6 +578,8 @@ class Dataset():
         else:
             urls = self.get_data(indices).apply_funcs([(apply_2D_ROI, [origin, size, center])],
                                                       save=roi_dir + "/data_roi", text="Applying roi")
+            if urls is None:
+                return
             new_data = Data(urls, self.get_data(indices).metadata, self._in_memory)
         if indices is None:
             shape = list(self.data.shape)[:-2]
@@ -542,11 +628,16 @@ class Dataset():
             os.mkdir(shift_dir)
 
         data = self.get_data(indices, dimension)
+        self._lock.acquire()
+        self.operations_state[Operation.SHIFT] = 1
+        self._lock.release()
 
         io_utils.advancement_display(0, len(data), "Applying shift")
         if dimension is not None:
             urls = []
             for i in range(len(data)):
+                if not self.operations_state[Operation.SHIFT]:
+                    return
                 filename = shift_dir + "/data_shift" + str(i).zfill(4) + ".npy"
                 img = apply_shift(data[i], shift[:, i], shift_approach)
                 numpy.save(filename, img)
@@ -565,6 +656,8 @@ class Dataset():
         else:
             urls = []
             for i in range(len(data)):
+                if not self.operations_state[Operation.SHIFT]:
+                    return
                 filename = shift_dir + "/data_shift" + str(i).zfill(4) + ".npy"
                 img = apply_shift(data[i], shift[:, i], shift_approach)
                 numpy.save(filename, img)
@@ -575,6 +668,11 @@ class Dataset():
                 numpy.put(new_urls, indices, urls)
             else:
                 new_urls = numpy.array(urls)
+
+        self._lock.acquire()
+        self.operations_state[Operation.SHIFT] = 0
+        self._lock.release()
+
         data = Data(new_urls.reshape(self.data.urls.shape), self.data.metadata, in_memory=self._in_memory)
         return Dataset(_dir=shift_dir, data=data, dims=self.__dims, in_memory=self._in_memory)
 
@@ -598,7 +696,7 @@ class Dataset():
         shift = self.find_shift(dimension, h_max, h_step, indices=indices)
         return self.apply_shift(shift, dimension, indices=indices)
 
-    def _cascade_nmf(self, num_components, iterations, indices=None):
+    def _cascade_nmf(self, num_components, iterations, vstep=None, hstep=None, indices=None):
         """
         This method is used as a way to improve the speed of convergence of
         the NMF method. For this, it uses a cascade methology at every
@@ -610,7 +708,7 @@ class Dataset():
         This way, the number of iterations with big images can be diminished, and
         the method converges faster.
 
-        :param int num_components: Number of componentsto find.
+        :param int num_components: Number of components to find.
         :param array_like iterations: Array with number of iterations per step of the cascade.
             The size of the array sets the size of the cascade.
         :param Union[None, array_like] indices: If not None, apply method only to indices of data.
@@ -632,13 +730,19 @@ class Dataset():
 
         for i in range(len(iterations)):
             new_urls = []
-            for j in range(len(self.get_data())):
+            if indices is None:
+                indices = range(self.nframes)
+            for j in indices:
                 filename = self.dir + "/cascade/" + str(j) + ".npy"
                 numpy.save(filename, resize(self.get_data(j), size))
                 new_urls.append(DataUrl(file_path=filename, scheme='fabio'))
-            data = Data(new_urls, self.get_data().metadata, self._in_memory)
+            data = Data(new_urls, self.get_data(indices).metadata, self._in_memory)
             dataset = Dataset(_dir=self.dir + "/cascade", data=data, in_memory=self._in_memory)
-            H, W = dataset.nmf(num_components, iterations[i], W=W, H=H, indices=indices)
+            if vstep:
+                v_step = vstep * (len(iterations) - i)
+            if hstep:
+                h_step = hstep * (len(iterations) - i)
+            H, W = dataset.nmf(num_components, iterations[i], W=W, H=H, vstep=v_step, hstep=h_step)
             size = first_size * (i + 2)
             H2 = numpy.empty((H.shape[0], size[0] * size[1]))
             for row in range(H.shape[0]):
@@ -650,6 +754,8 @@ class Dataset():
         except Exception as e:
             print('Failed to delete %s. Reason: %s' % (self.dir + "/cascade", e))
 
+        H = resize(H, (num_components, shape[0] * shape[1]))
+
         return H, W
 
     def pca(self, num_components=None, chunk_size=None, indices=None, return_vals=False):
@@ -678,22 +784,30 @@ class Dataset():
             os.mkdir(bss_dir)
         if self._in_memory:
             model = decomposition.PCA(n_components=num_components)
-            if indices is not None:
-                W = model.fit_transform(self.data.convert_to_hdf5(bss_dir)[indices])
-            else:
-                W = model.fit_transform(self.data.convert_to_hdf5(bss_dir)[:, :])
+            try:
+                if indices is not None:
+                    W = model.fit_transform(self.data.convert_to_hdf5(bss_dir)[indices])
+                else:
+                    W = model.fit_transform(self.data.convert_to_hdf5(bss_dir)[:, :])
+            finally:
+                self.data._file.close()
+
             H, vals, W = model.components_, model.singular_values_, W
         else:
             if chunk_size is None:
                 chunk_size = int(self.get_data().shape[0] / 10)
             model = IPCA(self.data.convert_to_hdf5(bss_dir), chunk_size, num_components, indices=indices)
             file = h5py.File(bss_dir + '/ipca.hdf5', 'w')
-            file["W"] = numpy.random.random((model.num_samples, model.num_components))
-            file["H"] = numpy.random.random((model.num_components, model.num_features))
-            model.fit_transform(W=file["W"], H=file["H"])
+            H, W, vals = None, None, None
+            try:
+                file["W"] = numpy.random.random((model.num_samples, model.num_components))
+                file["H"] = numpy.random.random((model.num_components, model.num_features))
+                model.fit_transform(W=file["W"], H=file["H"])
+            finally:
+                file.close()
+                self.data._file.close()
+
             H, vals, W = model.H, model.singular_values, model.W
-            file.close()
-        self.data._file.close()
         return vals if return_vals else (H, W)
 
     def nica(self, num_components, chunksize=None, num_iter=500, error_step=None, indices=None):
@@ -723,13 +837,14 @@ class Dataset():
         if self._in_memory:
             chunksize = None
         model = NICA(self.data.convert_to_hdf5(bss_dir), num_components, chunksize, indices=indices)
-        model.fit_transform(max_iter=num_iter, error_step=error_step)
-        self.data._file.close()
-
-        return model.H, model.W
+        try:
+            model.fit_transform(max_iter=num_iter, error_step=error_step)
+            return model.H, model.W
+        finally:
+            self.data._file.close()
 
     def nmf(self, num_components, num_iter=100, error_step=None, cascade=None,
-            H=None, W=None, indices=None):
+            H=None, W=None, vstep=100, hstep=1000, indices=None):
         """
         Compute Non-negative Matrix Factorization on the data.
         The method, first converts, if not already, the data into an hdf5 file object
@@ -769,28 +884,32 @@ class Dataset():
             if numpy.any(X[:, :] < 0):
                 _logger.warning("Setting negative values to 0 to compute NMF")
                 X[X[:, :] < 0] = 0
-            W = model.fit_transform(X, W=W, H=H)
-            self.data._file.close()
-            return model.components_, W
+            try:
+                W = model.fit_transform(X, W=W, H=H)
+                return model.components_, W
+            finally:
+                self.data._file.close()
         else:
             if cascade is not None:
-                H, W = self._cascade_nmf(num_components, cascade, indices=indices)
+                H, W = self._cascade_nmf(num_components, cascade, vstep, hstep, indices=indices)
 
             model = NMF(self.data.convert_to_hdf5(bss_dir), num_components, indices=indices)
-            model.fit_transform(max_iter=num_iter, H=H, W=W)
-            self.data._file.close()
-
-            return model.H, model.W
+            try:
+                model.fit_transform(max_iter=num_iter, H=H, W=W, vstep=vstep, hstep=hstep)
+                return model.H, model.W
+            finally:
+                self.data._file.close()
 
     def nica_nmf(self, num_components, chunksize=None, num_iter=500, cascade=None,
-                 error_step=None, indices=None):
+                 error_step=None, vstep=100, hstep=1000, indices=None):
         """
         Applies both NICA and NMF to the data. The init H and W for NMF are the
         result of NICA.
         """
         H, W = self.nica(num_components, chunksize, num_iter, error_step, indices=indices)
 
-        return self.nmf(num_components, num_iter, error_step, cascade, H, W, indices=indices)
+        return self.nmf(num_components, num_iter, error_step, cascade, H, W,
+                        vstep, hstep, indices=indices)
 
     def __deepcopy__(self, memo):
         """
@@ -815,14 +934,17 @@ class Data(numpy.ndarray):
     :param in_memory: If True, the data is loaded into memory, default True
     :type in_memory: bool, optional
     """
-    def __new__(cls, urls, metadata, in_memory=True):
+    def __new__(cls, urls, metadata, in_memory=True, data=None):
         urls = numpy.asarray(urls)
         if in_memory:
-            # Create array as stack of images
-            input_data = []
-            for url in urls.flatten():
-                input_data += [utils.get_data(url)]
-            input_data = numpy.asarray(input_data)
+            if data is not None and urls.shape == data.shape[:-2]:
+                input_data = data
+            else:
+                # Create array as stack of images
+                input_data = []
+                for url in urls.flatten():
+                    input_data += [utils.get_data(url)]
+                input_data = numpy.asarray(input_data)
             shape = list(urls.shape)
             shape.append(input_data.shape[-2])
             shape.append(input_data.shape[-1])
@@ -836,6 +958,9 @@ class Data(numpy.ndarray):
         obj.metadata = numpy.asarray(metadata)
         obj._file = None
 
+        obj._lock = threading.Lock()
+        obj.operations = numpy.zeros(len(Operation))
+
         return obj
 
     def __array_finalize__(self, obj):
@@ -873,6 +998,20 @@ class Data(numpy.ndarray):
             else:
                 return Data(self.urls[indices], self.metadata[indices], self.in_memory)
 
+    def stop_operation(self, operation):
+        """
+        Method used for cases where threads are created to apply functions to the data.
+        If method is called, the flag concerning the stop is set to 0 so that if the concerned
+        operation is running in another thread it knows to stop.
+
+        :param int operation: operation to stop
+        :type int: Union[int, `Operation`]
+        """
+        if hasattr(self, 'operations') and self.operations[operation]:
+            self._lock.acquire()
+            self.operations[operation] = 0
+            self._lock.release()
+
     @property
     def shape(self):
         """
@@ -887,7 +1026,14 @@ class Data(numpy.ndarray):
             shape.append(data.shape[1])
             return tuple(shape)
 
-    def apply_funcs(self, funcs=[], indices=None, save=False, text=""):
+    @property
+    def ndim(self):
+        if self.in_memory:
+            return super(Data, self).ndim
+        else:
+            return super(Data, self).ndim + 2
+
+    def apply_funcs(self, funcs=[], indices=None, save=False, text="", operation=None):
         """
         Method that applies a series of functions into the data. It can save the images
         into disk or return them.
@@ -900,6 +1046,9 @@ class Data(numpy.ndarray):
         :type indices: Union[None, array_like], optional
         :param save: If True, saves the images into disk, defaults to False
         :type save: bool
+        :param str text: Text to show in the advancement display.
+        :param int operation: operation to stop
+        :type int: Union[int, `Operation`]
 
         :returns: Array with the new urls (if data was saved)
         """
@@ -908,8 +1057,22 @@ class Data(numpy.ndarray):
         if isinstance(indices, int):
             indices = [indices]
         urls = []
-        io_utils.advancement_display(0, self.flatten().shape[0], text)
+        io_utils.advancement_display(0, len(self.urls.flatten()), text)
+
+        if not hasattr(self, 'operations'):
+            self.operations = numpy.zeros(len(Operation))
+            self._lock = threading.Lock()
+        self._lock.acquire()
+        self.operations[operation] = 1
+        self._lock.release()
         for i in indices:
+            if operation is not None and not self.operations[operation]:
+                if save:
+                    for j in indices:
+                        while j != i:
+                            filename = save + str(j).zfill(4) + ".npy"
+                            os.remove(filename)
+                return
             img = self[int(i)]
             for f, args in funcs:
                 img = f(*([img] + args))
@@ -917,7 +1080,10 @@ class Data(numpy.ndarray):
                 filename = save + str(i).zfill(4) + ".npy"
                 numpy.save(filename, img)
                 urls.append(DataUrl(file_path=filename, scheme='fabio'))
-            io_utils.advancement_display(i + 1, self.flatten().shape[0], text)
+            io_utils.advancement_display(i + 1, len(self.urls.flatten()), text)
+        self._lock.acquire()
+        self.operations[operation] = 0
+        self._lock.release()
         return numpy.array(urls)
 
     def save(self, path, indices=None):
@@ -977,8 +1143,11 @@ class Data(numpy.ndarray):
         :type shape: int or tuple of ints.
         :return: new Data object with urls and metadata reshaped to shape.
         """
+        data = None
+        if self.in_memory:
+            data = super(Data, self).reshape(shape).view(numpy.ndarray)
         return Data(self.urls.reshape(shape[:-2]), self.metadata.reshape(shape[:-2]),
-                    self.in_memory)
+                    self.in_memory, data=data)
 
     def take(self, indices, axis=None):
         """
@@ -994,15 +1163,21 @@ class Data(numpy.ndarray):
         """
         urls = numpy.take(self.urls, indices, axis)
         metadata = numpy.take(self.urls, indices, axis)
-
-        return Data(urls, metadata, self.in_memory)
+        data = None
+        if self.in_memory:
+            data = numpy.take(self, indices, axis).view(numpy.ndarray)
+        return Data(urls, metadata, self.in_memory, data=data)
 
     def flatten(self):
         """
         :return: new data with flattened urls and metadata (but not images).
         :rtype: :class:`Data`
         """
-        return Data(self.urls.flatten(), self.metadata.flatten(), self.in_memory)
+        data = None
+        urls = self.urls.flatten()
+        if self.in_memory:
+            data = super(Data, self).reshape((len(urls), self.shape[-2], self.shape[-1])).view(numpy.ndarray)
+        return Data(urls, self.metadata.flatten(), self.in_memory, data=data)
 
 
 class AcquisitionDims(object):
@@ -1080,6 +1255,25 @@ class AcquisitionDims(object):
                                           kind=self.__dims[axis].kind,
                                           size=size)
 
+    def to_dict(self):
+        dims = {}
+        if self.ndim == 0:
+            return
+        else:
+            for axis, dim in self.__dims.items():
+                assert isinstance(dim, Dimension)
+                dims[axis] = dim.to_dict()
+            return dims
+
+    def from_dict(self, _dict):
+        """
+        Convert dictionary of dimensions.
+        """
+        if len(_dict) > 0:
+            for _axis, _dim in _dict.items():
+                assert type(_dim) is dict
+                self.add_dim(_axis, Dimension.from_dict(_dim))
+
     def __iter__(self):
         for iAxis, dim in self.__dims.items():
             yield (iAxis, dim)
