@@ -94,7 +94,7 @@ class DimensionMapping(qt.QWidget):
         """
 
         :param axis: which axis is defining this dimension
-        :param :class:Dim dim: definition of the dimension to add
+        :param `Dimension` dim: definition of the dimension to add
         """
         if axis is None:
             axis = self._getNextFreeAxis()
@@ -112,7 +112,7 @@ class DimensionMapping(qt.QWidget):
         """
         Remove dimension.
 
-        :param Union[int,_DimensionItem]: row or item to remove
+        :param Union[int,`_DimensionItem`]: row or item to remove
         """
         if isinstance(row, _DimensionItem):
             iRow = row._row
@@ -131,6 +131,7 @@ class DimensionMapping(qt.QWidget):
             new_row = row - 1
             assert new_row >= 0
             widget.embedInTable(table=self._table, row=new_row)
+            widget.axis = self._getNextFreeAxis()
             self._dims[new_row] = widget
             del self._dims[row]
 
@@ -156,6 +157,10 @@ class DimensionWidget(DimensionMapping):
     def __init__(self, parent=None):
         DimensionMapping.__init__(self, parent)
         self._dataset = None
+        self._update_dataset = None
+        self.indices = None
+        self.bg_indices = None
+        self.bg_dataset = None
         self.metadata_type = _METADATA_TYPES['default']
         self.tolerance = 1e-9
 
@@ -202,6 +207,9 @@ class DimensionWidget(DimensionMapping):
     def dataset(self):
         return self._dataset
 
+    def getDataset(self):
+        return self._update_dataset, self.indices, self.bg_indices, self. bg_dataset
+
     def _updateKind(self, metadata_type=None):
         if metadata_type is None:
             metadata_type = self._metadataTypeCB.currentText()
@@ -216,18 +224,22 @@ class DimensionWidget(DimensionMapping):
         super().clear()
         self.dataset.clear_dims()
 
-    def setDataset(self, dataset):
+    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
 
         :param dataset: the dataset for which we want to define the
                            dimensions.
-        :type dataset: :class:`Dataset`
+        :type dataset: `Dataset`
         """
         assert isinstance(dataset, Dataset)
         self._dataset = dataset
-        if len(self.dataset.metadata) > 0:
+        self._update_dataset = dataset
+        self.indices = indices
+        self.bg_indices = bg_indices
+        self.bg_dataset = bg_dataset
+        if dataset.nframes > 0:
             for widget in self._dims.values():
-                widget._setMetadata(self.dataset.metadata[0])
+                widget._setMetadata(self.dataset.get_data().metadata[0])
 
     def _find_dimensions(self):
         """
@@ -246,8 +258,8 @@ class DimensionWidget(DimensionMapping):
         """
         Fit dimensions into the data.
 
-        :return : return status of the fit and fail reason, if any
-        :rtype: (bool str or None)
+        :returns: return status of the fit and fail reason, if any
+        :rtype: Union[bool,str,None]
         """
         qt.QApplication.setOverrideCursor(qt.Qt.WaitCursor)
         if self.dataset is None:
@@ -261,15 +273,16 @@ class DimensionWidget(DimensionMapping):
                 for dimension in self.dims.values():
                     axis = dimension.axis
                     dimension = dimension.dim
-                    values = numpy.unique([data.get_value(kind=dimension.kind, name=dimension.name)[0] for data in self.dataset.metadata])
-                    dimension.setUniqueValues(values)
+                    values = numpy.unique([data.get_value(kind=dimension.kind,
+                                           name=dimension.name)[0] for data in self.dataset.get_data().metadata])
+                    dimension.set_unique_values(values)
                     self.dataset.add_dim(axis=axis, dim=dimension)
             else:
                 raise ValueError("Cannot fit without dimensions")
             super().clear()
             for dim in self.dataset.dims:
                 self.addDim(dim[0], dim[1])
-            self.dataset.reshape_data()
+            self._update_dataset = self.dataset.reshape_data()
         except Exception as e:
             msg.setIcon(qt.QMessageBox.Warning)
             msg.setText("Error: {0}".format(e))
@@ -290,7 +303,7 @@ class DimensionWidget(DimensionMapping):
     def setDims(self, dims):
         """
 
-        :param dict dims: axis as key and :class:`Dimension` as value.
+        :param dict dims: axis as key and `Dimension` as value.
         """
         self.clear()
         assert type(dims) is dict
@@ -298,21 +311,23 @@ class DimensionWidget(DimensionMapping):
             assert type(axis) is int
             assert isinstance(dim, Dimension)
             self.addDim(axis=axis, dim=dim)
-            if self.dataset is not None and len(self.dataset.metadata) > 0:
+            if self.dataset is not None and len(self.dataset.data.metadata) > 0:
                 if not dim.unique_values:
-                    values = numpy.unique([data.get_value(kind=dim.kind, name=dim.name)[0] for data in self.dataset.metadata])
-                    dim.setUniqueValues(values)
+                    values = numpy.unique(
+                        [data.get_value(kind=dim.kind, name=dim.name)[0]
+                         for data in self.dataset.data.metadata])
+                    dim.set_unique_values(values)
             self.dataset.add_dim(axis=axis, dim=dim)
 
     def addDim(self, axis=None, dim=None):
         """
 
         :param axis: which axis is defining this dimension
-        :param :class:`Dimension` dim: definition of the dimension to add
+        :param `Dimension` dim: definition of the dimension to add
         """
         widget = super().addDim(axis, dim)
-        if self.dataset is not None and len(self.dataset.metadata) > 0:
-            widget._setMetadata(self.dataset.metadata[0])
+        if self.dataset is not None and len(self.dataset.data.metadata) > 0:
+            widget._setMetadata(self.dataset.data.metadata[0])
 
         return widget
 
@@ -439,9 +454,9 @@ class _DimensionItem(Dimension, qt.QWidget):
         self._toleranceLE.editingFinished.connect(_callback)
 
         # update values from `Dim` class
-        self._kindCB.currentTextChanged.connect(self._setKind)
-        self._namesCB.currentTextChanged.connect(self._setName)
-        self._sizeWidget.valueChanged.connect(self._setSize)
+        self._kindCB.currentTextChanged.connect(self.set_kind)
+        self._namesCB.currentTextChanged.connect(self.set_name)
+        self._sizeWidget.valueChanged.connect(self.set_size)
 
         self.embedInTable(table=table, row=row)
         self.__row = row
@@ -462,7 +477,7 @@ class _DimensionItem(Dimension, qt.QWidget):
         qt.QMessageBox.information(self, title, ', '.join(unique_values_str))
 
     def setUniqueValues(self, values):
-        super().setUniqueValues(values)
+        super().set_unique_values(values)
         if values is None:
             self._infoButton.hide()
         else:
@@ -518,7 +533,7 @@ class _DimensionItem(Dimension, qt.QWidget):
         return float(self._toleranceLE.text())
 
     def _setSize(self, size):
-        Dimension._setSize(self, size)
+        Dimension.set_size(self, size)
         self._sizeWidget.setValue(size=size)
 
     def setNames(self, names):
