@@ -35,6 +35,7 @@ from silx.gui import icons
 from silx.gui.colors import Colormap
 from silx.gui.plot import ScatterView, StackView, Plot1D
 from silx.gui.widgets.FrameBrowser import HorizontalSliderWithBrowser
+import silx.resources
 
 import darfix
 from darfix.io.utils import write_components
@@ -51,11 +52,8 @@ class DisplayComponentsWidget(qt.QMainWindow):
         # Widget with the type of bss and the number of components to compute
         widget = qt.QWidget(self)
         self._sv_components = StackView(parent=self)
-        self._sv_components.setColormap(Colormap(
-            name=darfix.config.DEFAULT_COLORMAP_NAME,
-            normalization='linear',
-            vmin=None,
-            vmax=None))
+        self._sv_components.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
+                                        normalization=darfix.config.DEFAULT_COLORMAP_NORM))
         self._sv_components.setGraphTitle("Components")
         self._sv_components.sigFrameChanged.connect(self._setCurve)
 
@@ -69,6 +67,8 @@ class DisplayComponentsWidget(qt.QMainWindow):
 
         self._toolbar = qt.QToolBar(parent=self)
         self._toolbar.setIconSize(self._toolbar.iconSize() * 1.2)
+        if not silx.resources.is_dir("darfix"):
+            silx.resources.register_resource_directory("darfix", "darfix.resources")
         curves_icon = icons.getQIcon('darfix:gui/icons/curves')
         scatter_icon = icons.getQIcon('darfix:gui/icons/scatter')
         self.curves_action = qt.QAction(curves_icon, "Curves", self)
