@@ -26,15 +26,21 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "11/05/2020"
+__date__ = "15/09/2020"
 
 
+import unittest
+import Orange
 from silx.gui import qt
 from orangecontrib.darfix.test.orangeWorkflowTest import OrangeWorkflowTest
-
+try:
+    from Orange.canvas.application.canvasmain import CanvasMainWindow
+except:
+    CanvasMainWindow = None
 app = qt.QApplication.instance() or qt.QApplication([])
 
 
+@unittest.skipUnless(CanvasMainWindow, "Only testing with orange versions older than 3.23.0")
 class TestWorkflow(OrangeWorkflowTest):
     """
     Create workflow to test that the widgets are correctly displayed in the canvas.
