@@ -26,13 +26,12 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "14/08/2019"
+__date__ = "16/04/2020"
 
 
 from silx.gui.colors import Colormap
 from silx.gui.plot import Plot2D
 from Orange.widgets.widget import OWWidget, Input
-from darfix.core.dataset import Dataset
 from darfix.gui.utils import ChooseDimensionWidget
 
 
@@ -48,7 +47,7 @@ class ZSumWidgetOW(OWWidget):
 
     # Inputs
     class Inputs:
-        dataset = Input("dataset", Dataset)
+        dataset = Input("dataset", tuple)
 
     def __init__(self):
         super().__init__()
@@ -66,18 +65,18 @@ class ZSumWidgetOW(OWWidget):
     def setDataset(self, dataset):
         # Make sum of dataset data
         if dataset:
-            self.dataset = dataset
-            image = dataset.data.sum(axis=0)
+            self.dataset = dataset[0]
+            image = self.dataset.zsum()
             self._plot.addImage(image)
             self.open()
-            if dataset.reshaped_data is not None:
+            if len(self.dataset.data.shape) > 3:
                 self._chooseDimensionWidget.show()
-                self._chooseDimensionWidget.setDimensions(dataset.dims)
+                self._chooseDimensionWidget.setDimensions(self.dataset.dims)
         else:
             self._plot.clear()
 
     def _filterStack(self, dim=0, val=0):
-        data = self.dataset.get_reshaped_data(dim, val)
+        data = self.dataset.get_data(dimension=[dim, val])
         if data.shape[0]:
             image = data.sum(axis=0)
             self._plot.addImage(image)
@@ -85,5 +84,5 @@ class ZSumWidgetOW(OWWidget):
             self._plot.clear()
 
     def _wholeStack(self):
-        image = self.dataset.data.sum(axis=0)
+        image = self.dataset.get_data().sum(axis=0)
         self._plot.addImage(image)
